package sun.security.jgss;

import sun.security.action.GetBooleanAction;
import java.security.PrivilegedActionException;
import java.security.AccessControlContext;
import java.security.PrivilegedExceptionAction;
import java.util.Vector;
import sun.security.jgss.spi.GSSCredentialSpi;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import javax.security.auth.login.LoginException;
import javax.security.auth.login.LoginContext;
import sun.net.www.protocol.http.spnego.NegotiateCallbackHandler;
import com.sun.security.auth.callback.TextCallbackHandler;
import java.security.Security;
import java.util.Iterator;
import javax.security.auth.kerberos.KerberosKey;
import javax.security.auth.kerberos.KerberosTicket;
import sun.security.jgss.spnego.SpNegoCredElement;
import java.util.Set;
import sun.security.jgss.spi.GSSNameSpi;
import javax.security.auth.kerberos.KerberosPrincipal;
import sun.security.jgss.krb5.Krb5NameElement;
import java.util.HashSet;
import javax.security.auth.Subject;
import org.ietf.jgss.GSSCredential;
import org.ietf.jgss.GSSName;
import org.ietf.jgss.GSSException;
import org.ietf.jgss.Oid;

public class GSSUtil
{
    public static final Oid GSS_KRB5_MECH_OID;
    public static final Oid GSS_KRB5_MECH_OID2;
    public static final Oid GSS_KRB5_MECH_OID_MS;
    public static final Oid GSS_SPNEGO_MECH_OID;
    public static final Oid NT_GSS_KRB5_PRINCIPAL;
    private static final String DEFAULT_HANDLER = "auth.login.defaultCallbackHandler";
    static final boolean DEBUG;
    
    public GSSUtil() {
    }
    
    static void debug(String x) {
        if (GSSUtil.DEBUG) {
            assert x != null;
            System.out.println(x);
        }
    }
    
    public static Oid createOid(String s) {
        try {
            return new Oid(s);
        }
        catch (GSSException ex) {
            debug("Ignored invalid OID: " + s);
            return null;
        }
    }
    
    public static boolean isSpNegoMech(Oid other) {
        return GSSUtil.GSS_SPNEGO_MECH_OID.equals(other);
    }
    
    public static boolean isKerberosMech(Oid other) {
        return GSSUtil.GSS_KRB5_MECH_OID.equals(other) || GSSUtil.GSS_KRB5_MECH_OID2.equals(other) || GSSUtil.GSS_KRB5_MECH_OID_MS.equals(other);
    }
    
    public static String getMechStr(Oid oid) {
        if (isSpNegoMech(oid)) {
            return "SPNEGO";
        }
        if (isKerberosMech(oid)) {
            return "Kerberos V5";
        }
        return oid.toString();
    }
    
    public static Subject getSubject(GSSName obj, GSSCredential gssCredential) {
        HashSet<Object> set = new HashSet();
        HashSet<KerberosPrincipal> set2 = new HashSet();
        if ((obj instanceof GSSNameImpl)) {
            try {
                GSSNameSpi element = ((GSSNameImpl)obj).getElement(GSSUtil.GSS_KRB5_MECH_OID);
                String name = element.toString();
                if ((element instanceof Krb5NameElement)) {
                    name = ((Krb5NameElement)element).getKrb5PrincipalName().getName();
                }
                set2.add(new KerberosPrincipal(name));
            }
            catch (GSSException obj2) {
                debug("Skipped name " + obj + " due to " + obj2);
            }
        }
        Object o;
        if ((gssCredential instanceof GSSCredentialImpl)) {
            Set<GSSCredentialSpi> elements = ((GSSCredentialImpl)gssCredential).getElements();
            o = new HashSet(elements.size());
            populateCredentials(o, elements);
        }
        else {
            o = new HashSet();
        }
        debug("Created Subject with the following");
        debug("principals=" + set2);
        debug("public creds=" + set);
        debug("private creds=" + o);
        return new Subject(false, set2, set, o);
    }
    
    private static void populateCredentials(Set<Object> set, Set<?> set2) {
        for (Object obj : set2) {
            if ((obj instanceof SpNegoCredElement)) {
                obj = ((SpNegoCredElement)obj).getInternalCred();
            }
            if ((obj instanceof KerberosTicket)) {
                if (!obj.getClass().getName().equals("javax.security.auth.kerberos.KerberosTicket")) {
                    KerberosTicket kerberosTicket = (KerberosTicket)obj;
                    obj = new KerberosTicket(kerberosTicket.getEncoded(), kerberosTicket.getClient(), kerberosTicket.getServer(), kerberosTicket.getSessionKey().getEncoded(), kerberosTicket.getSessionKeyType(), kerberosTicket.getFlags(), kerberosTicket.getAuthTime(), kerberosTicket.getStartTime(), kerberosTicket.getEndTime(), kerberosTicket.getRenewTill(), kerberosTicket.getClientAddresses());
                }
                set.add(obj);
            }
            else if ((obj instanceof KerberosKey)) {
                if (!obj.getClass().getName().equals("javax.security.auth.kerberos.KerberosKey")) {
                    KerberosKey kerberosKey = (KerberosKey)obj;
                    obj = new KerberosKey(kerberosKey.getPrincipal(), kerberosKey.getEncoded(), kerberosKey.getKeyType(), kerberosKey.getVersionNumber());
                }
                set.add(obj);
            }
            else {
                debug("Skipped cred element: " + obj);
            }
        }
    }
    
    public static Subject login(GSSCaller gssCaller, Oid oid) throws LoginException {
        NegotiateCallbackHandler callbackHandler;
        if ((gssCaller instanceof HttpCaller)) {
            callbackHandler = new NegotiateCallbackHandler(((HttpCaller)gssCaller).info());
        }
        else {
            String property = Security.getProperty("auth.login.defaultCallbackHandler");
            if (property != null && property.length() != 0) {
                callbackHandler = null;
            }
            else {
                callbackHandler = new TextCallbackHandler();
            }
        }
        LoginContext o = new LoginContext("", null, callbackHandler, new LoginConfigImpl(gssCaller, oid));
        o.login();
        return o.getSubject();
    }
    
    public static boolean useSubjectCredsOnly(GSSCaller gssCaller) {
        String privilegedGetProperty = GetPropertyAction.privilegedGetProperty("javax.security.auth.useSubjectCredsOnly");
        if ((gssCaller instanceof HttpCaller)) {
            return "true".equalsIgnoreCase(privilegedGetProperty);
        }
        return !"false".equalsIgnoreCase(privilegedGetProperty);
    }
    
    public static boolean useMSInterop() {
        return !((String)AccessController.doPrivileged(new GetPropertyAction("sun.security.spnego.msinterop", "true"))).equalsIgnoreCase("false");
    }
    
    public static <T extends GSSCredentialSpi> Vector<T> searchSubject(final GSSNameSpi gssNameSpi, final Oid oid, final boolean b, final Class<? extends T> clazz) {
        debug("Search Subject for " + getMechStr(oid) + (b ? " INIT" : " ACCEPT") + " cred (" + ((gssNameSpi == null) ? "<<DEF>>" : gssNameSpi.toString()) + ", " + clazz.getName() + ")");
        final AccessControlContext context = AccessController.getContext();
        try {
            return (Vector)AccessController.doPrivileged(new PrivilegedExceptionAction<Vector<T>>() {
                @Override
                public Vector<T> run() throws Exception {
                    Subject subject = Subject.getSubject(context);
                    Vector<Object> vector = null;
                    if (subject != null) {
                        vector = new Vector();
                        for (GSSCredentialImpl obj : subject.getPrivateCredentials(GSSCredentialImpl.class)) {
                            GSSUtil.debug("...Found cred" + obj);
                            try {
                                GSSCredentialSpi element = obj.getElement(oid, b);
                                GSSUtil.debug("......Found element: " + element);
                                if (element.getClass().equals(clazz) && (gssNameSpi == null || gssNameSpi.equals(element.getName()))) {
                                    vector.add(clazz.cast(element));
                                }
                                else {
                                    GSSUtil.debug("......Discard element");
                                }
                            }
                            catch (GSSException obj2) {
                                GSSUtil.debug("...Discard cred (" + obj2 + ")");
                            }
                        }
                        return vector;
                    }
                    GSSUtil.debug("No Subject");
                    return vector;
                }
            });
        }
        catch (PrivilegedActionException ex) {
            debug("Unexpected exception when searching Subject:");
            if (GSSUtil.DEBUG) {
                ex.printStackTrace();
            }
            return null;
        }
    }
    
    static {
        GSS_KRB5_MECH_OID = createOid("1.2.840.113554.1.2.2");
        GSS_KRB5_MECH_OID2 = createOid("1.3.5.1.5.2");
        GSS_KRB5_MECH_OID_MS = createOid("1.2.840.48018.1.2.2");
        GSS_SPNEGO_MECH_OID = createOid("1.3.6.1.5.5.2");
        NT_GSS_KRB5_PRINCIPAL = createOid("1.2.840.113554.1.2.2.1");
        DEBUG = (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.security.jgss.debug"));
    }
}
